{{------------------------------------------------------------------------------
Book_PASM_Read_1pot.SPIN 
Function of program
Reads one Potentiometer and displays on PST

Connections are as follows:
Pots on standard 3208 connections 24..27
LED on 19

Program         Book_PASM_Read_1pot.SPIN
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Dec 21 2012

Revisions:      Report errors to harprit.sandhu@gmail.com

-------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
    pin=19
VAR
  long p_val2
  long stack2[25]                'space for Cog_PST
    
OBJ 
  fds : "FullDuplexSerial"
  
PUB Go | P_VAL                   'P_VAL is declared local here
fds.start(31,30,0,115200)        'start console at 115200 for debug output
  cognew(@generate, @P_Val)      'new Cog at "generate" and read var at P_Val
  cognew(osco, @stack2)
  dira[pin]~~
  repeat
    P_VAL2:=P_VAL                  'endless loop to display data
    outa[0..11] := P_Val           'displays 1.5 bytes of data
    fds.bin(P_val,12)              'print value
    fds.tx($d)                     'new line
    fds.dec(P_val)                 'print value
    fds.tx(" ")                    'print space
    fds.tx($d)                     'new line
    fds.tx(1)                      'home
    waitcnt(clkfreq/60+cnt)        'flicker free wait

PRI osco                             'Method identified
dira [pin]~~                          'make pin an output
  repeat                             'start loop
    !outa[pin]                        'invert pin
    waitcnt(clkfreq/(P_VAL2/100+5)+cnt)  'wait based on P_QAL.  
                                     '5 is added so value is always >5

DAT        org     0                         'sets the starting point in Cog
generate   mov     dira,         set_dira    'sets direction of the prop pins
           call    #chip_sel_lo              'selects chip by pulling line low
           call    #Clk_lo     'START. Clock needs to be low to load data
           call    #Din_hi     'must start with Din high to set up 3208
           call    #Tog_clk    'clk hi to read data
           call    #Din_Hi     'SINGLE DIFF  Low to load
           call    #Tog_Clk    'toggle clock line hi then low to read data
           call    #Din_Lo     'D2 Low to load input sequence 000 for line 0
           call    #Tog_Clk    'toggle clock line hi then low to read in the data
           call    #Din_Lo     'D1 Low to load input line sequence 000 for line 0
           call    #Tog_Clk    'toggle clock line hi then low to read in the data
           call    #Din_Lo     'D0 Low to load input line  sequence 000 for line 0
           call    #Tog_Clk    'toggle clock line hi then low to read in the data
           call    #Din_Lo     'blank bit needs a clock cycle, next
           call    #Tog_Clk    'toggle clock line hi then low to read in the data
                               'next toggle is for the null bit, nothing read
           call    #Tog_Clk    'toggle clock line hi then low to read in the data
           mov     dat_red,  #0  'Clear register we will read data into
           mov     count,    #12 'Counter for number of bits we will read
read_bit   mov     temp,     ina 'read in what is in all the input lines
           andn    temp,     inputmask wz   'mask off except Dout line. Set Z flag
           shl     Dat_red,  #1     'roll right 1 bit to get ready for next bit
     if_nz add     Dat_red,  #1     'if value positive add 1 to data register
           call    #Tog_Clk         'toggle clock to get next bit ready in Dout
           sub     count,    #1 wz  'decrement the "bits read" counter. Set Z flag
     if_nz jmp     #read_bit        'go up and do it again if counter not 0
           wrlong  dat_red,   par   'put it in PAR to share it as P.Val
           call    #Chip_Sel_Hi     'Put chip to sleep by del selecting it
           jmp     #generate        'go back to do it all again

'Subroutines==================================================================
Clk_Hi        or     outa,   clk_bit     'OR it with the Clock Bit to make high
Clk_Hi_ret           ret                 'return
'=======================
Clk_Lo        andn   outa ,   clk_bit    'ANDN it with the Clock Bi to make low
Clk_Lo_ret           ret                 'return from this subroutine
'=======================
Tog_Clk       call   #Clk_hi             'make clock bit high
              call   #clk_lo             'make clock bit low
Tog_Clk_ret          ret                 'return from this subroutine
'=======================
Din_Hi        or     outa ,   din_Bit    'Makes the Din high
Din_Hi_ret           ret                 'return from this subroutine
'=======================
Din_Lo        andn   outa ,   din_Bit    'makes Din low
Din_Lo_ret           ret                 'return from this subroutine
'=======================
Chip_Sel_Hi   or     outa ,   chs_Bit    'Makes Chip select high
Chip_Sel_Hi_ret      ret                 'return from this subroutine
'=======================
Chip_Sel_Lo   andn   outa,   chs_Bit     'makes chip select low
Chip_Sel_Lo_ret      ret                 'return from this subroutine
'=======================
Read_Dout     mov    temp,   ina         'Get the INA register
Read_Dout_ret        ret                 'return from this subroutine
'=======================
Read_Next_Bit mov       temp,  ina          'Get the INA register
              or        temp,  inputmask    'mask all but Din bit
Read_Next_Bit_ret       ret                 'return from this subroutine
'=======================
'Constants. This section is similar to the CON block in SPIN
Set_dira   long  %00001011_10000000_00011111_11111111   'Set dira register
Chs_Bit    long  %00000001_00000000_00000000_00000000   'Chip select bit  24
Din_Bit    long  %00000010_00000000_00000000_00000000   'Data in bit      25
Dout_Bit   long  %00000100_00000000_00000000_00000000   'Data out bit     26
Clk_Bit    long  %00001000_00000000_00000000_00000000   'Clock bit        27
inputmask  long  %11111011_11111111_11111111_11111111   'Mask for the Dout bit
Pin_23     long  %00000000_10000000_00000000_00000000   'Set dira register
'===============================================================================
'Variables. This section is similar to the VAR block in SPIN 
temp          res       1       'temporary storage variable, misc
count         res       1       'temporary storage variable, bit counter
Dat_Red       res       1       'temporary storage variable, data being read
Fit
