{{-------------------------------------------------------------------------------
PASM_Book_ReadPot.SPIN
Function of program
Makes an electronic Metronome.

Notes:  Program reads a potentiometer
See details in book

Program         PASM_Book_ReadPot.SPIN
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Jan 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com
  
--------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x   'set clock
  _xinfreq = 5_000_000        'crystal freq

VAR
  long P_Val                  'declare variable

OBJ 
  fds : "FullDuplexSerial"    'for display to PST

PUB Main  
fds.start(31,30,0,115200)     'start console at 115200 for debug output
  cognew(@generate, @P_Val)   'start new Cog at "generate" and read variable into P_Val in PASM
  repeat                      'loop
    fds.bin(P_val,12)         'print value to the PST in binary 
    fds.tx($d)                'new line 
    fds.dec(P_val)            'print value as decimal 
    fds.tx(" ")               'space       
    fds.tx($1)                'home to 0,0
    waitcnt(clkfreq/60+cnt)   'flicker free wait

DAT         org    0                 'sets the starting point in Cog
generate    mov    dira,   set_dira  'sets direction of the prop pins
            call   #chip_sel_lo      'selects chip by pulling line low 
            call   #Clk_lo           'START. Clock needs to be low to load data
            call   #Din_hi           'must start with Din high to set up 3208
            call   #Tog_clk          'clk hi-lo to read data
            call   #Din_Hi           'SINGLE DIFF mode, will be made Low to load
            call   #Tog_Clk          'toggle clock line hi low to read data                 
            call   #Din_Lo  'D2 Low to load input line sel sequence 000 for line 0
            call   #Tog_Clk 'toggle clock line hi then low to read in the data
            call   #Din_Lo  'D1 Low to load input line sel sequence 000 for line 0
            call   #Tog_Clk 'toggle clock line hi then low to read in the data 
            call   #Din_Lo  'D0 Low to load input line sel sequence 000 for line 0
            call   #Tog_Clk      'toggle clock line hi low to read in the data                 
            call   #Din_Lo       'blank bit needs a clock cycle, next
            call   #Tog_Clk      'toggle clock line hi then low to read the data
                                 'next toggle is for the null bit, nothing read
            call   #Tog_Clk      'toggle clock line hi then low to read the data
            mov    dat_red,  #0        'Clear register we will read data into
            mov    count,    #12       'Counter for number of bits we will read
read_bit    mov    temp,    ina        'read in what is in all the input lines
            andn   temp,    mask26 wz  'mask off except Dout line & Set Z flag
            shl    Dat_red,  #1     'shift register left 1 bit ready for next bit
      if_nz add    Dat_red,  #1     'if value is positive add 1 to data register
            call   #Tog_Clk         'toggle clock to get next bit ready in Dout
            sub    count,    #1 wz  'decrement "bits read" counter. Set Z flag
      if_nz jmp    #read_bit        'go up and do it again if counter not yet 0
            mov    mem,      par    'get address of par into mem
            wrlong dat_red,   mem   'write it in HUB to share it as P.Val
            call   #Chip_Sel_Hi     'Put chip to sleep by de-selecting it
            jmp    #generate        'go back to do it all again
 
'Subroutines
Clk_Hi        or      outa,   clk_bit  'OR it with the Clock Bit to make high
Clk_Hi_ret            ret              'return from this subroutine 

Clk_Lo        andn    outa ,   clk_bit 'ANDN it with the Clock Bi to make low
Clk_Lo_ret            ret              'return from this subroutine

Tog_Clk       call    #Clk_hi          'make clock bit high
              call    #clk_lo          'make clock bit low
Tog_Clk_ret           ret              'return from this subroutine

Din_Hi        or      outa ,   din_Bit 'Makes the Din high
Din_Hi_ret            ret              'return from this subroutine

Din_Lo        andn    outa ,   din_Bit 'makes Din low
Din_Lo_ret            ret              'return from this subroutine

Chip_Sel_Hi   or      outa ,   chs_Bit 'Makes Chip select high
Chip_Sel_Hi_ret       ret              'return from this subroutine

Chip_Sel_Lo   andn    outa,   chs_Bit  'makes chip select low
Chip_Sel_Lo_ret       ret              'return from this subroutine

Read_Next_Bit mov     temp,  ina       'Get the INA register
              or      temp,  mask26    'mask all but Din bit
Read_Next_Bit_ret     ret              'return from this subroutine
'======================
'Constants. This is similar to the CON block in SPIN but these are not constants
'because they can be changed.  These are initial values.
Set_dira   long   %00001011_11000000_00001111_11111111   'Set dira register
Chs_Bit    long   %00000001_00000000_00000000_00000000   'Chip select bit     24
Din_Bit    long   %00000010_00000000_00000000_00000000   'Data in bit         25
Dout_Bit   long   %00000100_00000000_00000000_00000000   'Data out bit        26
Clk_Bit    long   %00001000_00000000_00000000_00000000   'Clock bit           27
mask26     long   %11111011_11111111_11111111_11111111   'Mask forDout bit only
Pin_23     long   %00000000_10000000_00000000_00000000   'osc line
Pin_22     long   %00000000_01000000_00000000_00000000   'Speaker line
'============================
'Variables. This section is similar to the VAR block in SPIN. 
'These are blank variable space designators
temp        res    1    'temporary storage variable, misc
count       res    1    'read bit counter
Dat_Red     res    1    'data being read
mem         res    1    'par location
Fit
