{{--------------------------------------------------------------------------------
PASM_Book_ReadPot2.SPIN
Function of program
Makes an electronic Metronome.

Notes:  Program reads a potentiometer
See details in book

Program         PASM_Book_ReadPot2.SPIN
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Jan 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

---------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  bits=12
  pots=4
  
VAR
  long P_Val[8]
  byte  pot_sub
  
OBJ 
  fds : "FullDuplexSerial"
  
PUB Main                            'P_VAL is a local variable here
fds.start(31,30,0,115200)           'start console at 115200 for debug output
cognew(@generate, @P_Val)           'start new Cog at "generate" read var to P_Val
'waitcnt(clkfreq/8+cnt)             'wait
  repeat                            'endless loop
    pot_sub:=pots                   'set number of pots to be read
    fds.tx($1)                      'home to 0,0
    repeat pots                     'loop to display data
      fds.bin(P_val[pot_sub], bits) 'print to the PST in binary  
      fds.tx(" ")                   'space to erase old data overflow  
      fds.tx(" ")                   'space to erase old data overflow 
      fds.dec(P_val[pot_sub])       'print value as decimal value
      repeat 3                      'three spaces
        fds.tx(" ")                 'space to erase old data overflow 
      fds.tx($d)                    'new line
      pot_sub:=pot_sub-1            'decrement counter
    waitcnt(clkfreq/60+cnt)         'flicker free wait
        
DAT           org       0                   'sets the starting point in Cog
generate      mov       pot_count, #4       'number of pots to read [also change bits read]
              mov       bits_read, #12    'bits to be read
              mov       mem,      par     'get address of mem
              mov       dira,    set_dira 'sets direction of the prop pins
gen2          or        outa ,   chs_Bit  'Makes Chip select high
              andn      outa,    chs_Bit  'makes chip select low 
              andn      outa ,   clk_bit  'ANDN it with the Clock Bi to make low
              or        outa ,   din_Bit  'Makes the Din high
              call      #Tog_clk          'clk hi-lo to read data
              or        outa ,   din_Bit  'Makes the Din high 
              call      #Tog_Clk          'toggle clock line hi-low to read data
              call      #get_third_bit    'get bit
       if_nz  jmp       #its_one          'jump to set it as needed
              jmp       #its_zero         'jump to set it as needed
its_one       call      #Set_next_bit1    'setting bit subroutine call
              jmp       #continue1        'go to next bit
its_zero      call      #Set_next_bit0    'setting bit subroutine call
continue1     call   #get_second_bit      'get bit
       if_nz  jmp    #its_one1            'jump to set it as needed
              jmp    #its_zero1           'jump to set it as needed
its_one1      call   #Set_next_bit1       'setting bit subroutine call
              jmp    #continue2           'go to next bit
its_zero1     call   #Set_next_bit0       'setting bit subroutine call
continue2     call   #get_first_bit       'get bit
       if_nz  jmp    #its_one2            'jump to set it as needed
              jmp    #its_zero2           'jump to set it as needed
its_one2      call   #Set_next_bit1       'setting bit subroutine call
              jmp    #continue3           'go to next bit
its_zero2     call   #Set_next_bit0       'setting bit subroutine call
continue3     andn   outa ,   din_Bit     'makes Din low              
              call   #Tog_Clk             'toggle clock line hi-low to read data
              call   #Tog_Clk             'toggle clock line hi-low to read data
              mov    dat_red,  #0         'Clear register we will read data into 
              mov    count,    bits_read  'Counter for number of bits we will read
read_bit      mov    temp,     ina        'read in what is in all the input lines
              andn   temp,     mask26 wz  'mask off except Dout line. Set Z flag
              shl    Dat_red,  #1         'shift register left 1 bit for next bit
        if_nz add    Dat_red,  #1         'if value + add 1 to data register    
              call   #Tog_Clk             'toggle clock get next bit ready in Dout
              sub    count,    #1 wz      'decr the "bits read" counter. Set Z flag
        if_nz jmp    #read_bit            'go up do it again if counter not yet 0 
              wrlong Dat_red,  mem         'write it in PAR to share it as P.Val
              add    mem,  #4              'add 4 to HUB address for next long
              mov    temp,     pot_count wz  'get the pot number being looked at
              sub    pot_count, #1           'subtract 1 g=from it
      if_nz jmp      #gen2                   'if it is not 0 go up for next pot
              jmp    #generate               'go back to do it all again
'subroutines used
Set_next_bit0  andn  outa ,    din_Bit     'makes Din low   000 for line 0
              call   #Tog_Clk              'toggle clock line hi-low to read data
Set_next_bit0_ret    ret                   'return from this subroutine

Set_next_bit1 or     outa ,    din_Bit     'makes Din low 000 for line 0
              call   #Tog_Clk              'toggle clock line hi-low to read data 
Set_next_bit1_ret    ret                   'return from this subroutine

Tog_Clk       nop                              'nop to settle signals
              or        outa,      clk_bit     'make clock bit high
              nop                              'nop to settle signals
              andn      outa,      clk_bit     'make clock bit low
              nop                              'nop to settle signals
Tog_Clk_ret   ret                              'return from this subroutine 

Get_first_bit
               mov      temp,      pot_count   'get pot number
               andn     temp,      mask0 wz    'get last bit
Get_first_bit_ret       ret                    'return

Get_second_bit
               mov      temp,      pot_count   'get pot number
               shr      temp,      #1          'shift right 1 bit to get second bit
               andn     temp,      mask0 wz    'return   
Get_second_bit_ret      ret 

Get_third_bit 
               mov      temp,      pot_count   'get pot number
               shr      temp,      #2          'shift right 2 bits to get third bit
               andn     temp,      mask0 wz    'return 
Get_third_bit_ret        ret

Set_dira      long  %00001011_00000000_00000000_00000000   'Set dira register
Chs_Bit       long  %00000001_00000000_00000000_00000000   'Chip select bit   24
Din_Bit       long  %00000010_00000000_00000000_00000000   'Data in bit       25
Dout_Bit      long  %00000100_00000000_00000000_00000000   'Data out bit      26
Clk_Bit       long  %00001000_00000000_00000000_00000000   'Clock bit         27
mask26        long  %11111011_11111111_11111111_11111111   'Mask to read Dout bit
mask0         long  %11111111_11111111_11111111_11111110   'Mask to read LS bit
bits_Read      res  1       'A2D resolution in bits
temp           res  1       'temporary storage variable, misc
count          res  1       'temporary storage variable, read bit counter
Dat_Red        res  1       'temporary storage variable, data being read
mem            res  1       'memory location for PAR variable
pot_Count      res  1       'current pot number
Fit
