{{-------------------------------------------------------------------------------
Book_PASM_Frequency.SPIN
Function of program
This programs controls an R/C servo from a potentiometer.

Notes: The first 4 PAR locations are being used for reading the pots
so they cannot be used for other purposes in any other cogs.
A 3208 and one pot must be installed for this program
Look at waveform on line 0.  Pot value is displayed on PST

Program         Book_PASM_Frequency.SPIN
Book            PASM for beginners, Propeller 102
Section         3 Outputs
Programmer      Harprit Sandhu   
Date            Jan 20 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

---------------------------------------------------------------------------------}} 
CON
  _clkmode = XTAL1 + PLL16x
  _xinfreq = 5_000_000

VAR
  long  P_Val[4]   
   
OBJ 
  fds : "FullDuplexSerial"      'for communications with PST 
 r4p : "read4pots"              'reads pots 
  
PUB Main|x 
fds.start(31,30,0,115200)       'start console at 115200 for debug output
r4p.Read4Pots(@P_Val)           'start pot reading loop in read4pots object                         
cognew(@PCog, @P_Val)           'Cog in PASM for freq gen 
fds.tx(16)                      'clear screen
repeat         
    fds.tx(1)                   'home
    fds.dec(P_Val[0])           'print pot0 value
    fds.tx(" ")                 'space
    fds.tx($d)                  'next line

    waitcnt(clkfreq/60+cnt)     'flicker free wait
    x:=x+1           'increment counter. This routine clears up screen
    if x>100         'decision point     by erasing extra lines of bot
      fds.tx(16)     'clear screen       of PST display every 10 loops
      x:=0           'reset counter
' 
DAT       org     0                  'start at 0 location in the Cog 
PCog      mov     dira,   set_Dira   'set direction register
          mov     outa,   set_Pin    'set pin states
          mov     mem,    par        'read par
:loop     andn    outa,   mask0      'pin low
          call    #delay             'wait
          or      outa,   mask0      'pin high
          call    #delay             'wait
          jmp     #:loop             'go do it again
'=======================
'Subroutines
delay     rdlong  temp,   mem        'read first mem location which is pot 0
          add     temp,   #511       'add 50 to keep frequencies a bit lower
:loop     djnz    temp,   :loop      'decrement and jump
delay_ret ret
'=======================
'constants and variables 
Set_Dira    long    %00000000_00000000_00000000_00000001
Set_Pin     long    %00000000_00000000_00000000_00000001
mask0       long    %00000000_00000000_00000000_00000001

mem        res    1
select     res    1
temp       res    1
Fit
