{{------------------------------------------------------------------------------
Book_PASM_Advanced freq.SPIN
Function of program
Creating square waves using a inner and outer loop to
allow lower frequencies to be created and better control
of the frequency range to be implemented

Connections are as follows:
MCP3208 on 24..27  Uses pot0
Output on pin0

Program         Book_PASM_Advanced Freq.SPIN
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu
Date            Dec 21 2012

Revisions:      Report errors to harprit.sandhu@gmail.com

-------------------------------------------------------------------------------}}
CON
_clkmode = xtal1 + pll16x
_xinfreq = 5_000_000

VAR
  long  P_Val[10]

OBJ
  r4p : "read4pots" 
  fds : "FullDuplexSerial"           'for communications with PST 
  
PUB Go|x 
r4p.Read4Pots(@P_Val)               'start pot reading loop in read4pots object
waitcnt(clkfreq/100+cnt)            'wait
fds.start(31,30,0,115200)           'start console at 115200 for debug output
fds.tx(16)                          'clear screen
cognew(@toggle3, @P_val)            'start Cog                                                                                                          'clear screen
repeat         
    fds.tx(1)                       'home
    fds.str(string("Pot 0 =  "))    'print
    fds.dec(P_Val[0])               'print pot0 value
    fds.str(string("     "))        'spaces
    fds.tx($d)                      'next line
    fds.str(string("Counter  1  ")) 'print
    fds.dec(P_Val[5])               'print counter 1
    fds.str(string("     "))        'spaces
    fds.tx($d)                      'next line
    fds.str(string("Counter  2  ")) 'print
    fds.dec(P_Val[6])               'print counter 2
    fds.str(string("     "))        'spaces
    fds.tx($d)                      'next line
    fds.tx($d)                      'next line
    fds.str(string("Difference  ")) 'print
    fds.dec(||(P_Val[6]-P_Val[5]))  'print difference
    fds.str(string("     "))        'space
    fds.tx($d)                      'next line

    waitcnt(clkfreq/6+cnt)    'flicker free wait
    x:=x+1                    'increment counter. This routine clears up screen
    if x>100                  'decision point     by erasing extra lines of bot
      fds.tx(16)              'clear screen       of PST display every 10 loops
      x:=0                    'reset counter
    P_Val[4]:=1               'switch for PASM ON. Try commenting this out

DAT       org      0              'start of the program storage locations
toggle3   mov     mem,    Par
          add     mem,    #16
          mov     temp,   #0
          wrlong  temp,   mem
:loop9    rdlong  temp,   mem  wz
if_z      jmp     #:loop9       
loop6     mov     mem,    par     'read par register
          mov     mem1,   mem     'set mem1=mem
          add     mem1,   #20     'set to hub location 5
          mov     mem2,   mem     'set mem1=mem      
          add     mem2,   #24     'set to hub location 6
          mov     dira,   #%11    'pin now sets lines 0 and 1 as outputs
          mov     outa,   #%01    'sets the two bits 
:loop     call    #micro_Del      'call the delay subroutine
          shl     outa,   #%1     'shift left one bit
          mov     temp,   cnt     '1st reading of counter
          wrlong  temp,   mem1    'store in hub 5
          call    #micro_Del      'call the delay subroutine
          mov     temp,   cnt     'second reading of counter
          wrlong  temp,   mem2    'store in value 6
          shr     outa,   #%1     'shift back right one bit
          jmp     #:loop          'repeat loop
'============subroutines=====================
micro_Del rdlong  micro,  mem      'routine delays outer loop, val read from pot0
          add     micro,  #476     'add to increase loop time, arbitrary
:loop2    mov     micro2, #10      'load inner loop
:loop     djnz    micro2, #:loop   'completes inner loop
          djnz    micro,  #:loop2  'completes outer loop
micro_Del_ret ret                  'return
'=========constants and variables=============
temp      res     1        'time of delay
time      res     1        'location for time
mem       res     1        '
mem1      res     1        '
mem2      res     1        '
micro     res     1        '
micro2    res     1        '
Fit