{{--------------------------------------------------------------------------------
Book_PASM_ServoRC.SPIN  
Function of program
This programs controls an R/C servo from a potentiometer.

Notes:  In this program the first 4 PAR ref locations are being used by the pots
so they cannot be used for other purposes in any other cogs.
3208 and one pot must be installed
Servo white wire goes to line 0
Red wire goes to 5 volts
Black wire goes to ground

Program         Book_PASM_ServoRC.SPIN 
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Jan 20 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

---------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
VAR
  long P_Val[32]
  long  subsc
  long  clear
  
OBJ 
  r4p : "read4pots" 
  fds : "FullDuplexSerial"
  
PUB Go|x,lines,bits             'x is local variable used in this method
r4p.Read4Pots(@P_Val)'start pot reading loop read4pots object, MUST start first
fds.start(31,30,0,115200)       'start console at 115200 for debug output
waitcnt(clkfreq/60 +cnt)        'let all cogs start up
cognew(@timer60, @P_Val)        'start PASM Cog
lines:=6                        'display 6 variable on 6 lines
bits:=12                        'display to 12 bits (pots are read to 12 bits
  repeat 
    fds.tx($1)                  'home and print heading
    fds.str(string("Displays first "))
    fds.dec(lines)
    fds.str(string(" PAR/HUB locations to "))
    fds.dec(bits)    
    fds.str(string(" bits."))
    fds.str(string("    "))       'print spaces to erase old data overflow
    fds.tx($d)                    'new line   
    fds.tx($d)                    'new line
    repeat subsc from 0 to 12      'repeat for each line shown
      if subsc==4                 'if line 4
        fds.tx($d)                'insert extra line
      if subsc==10                'if line 
        fds.tx($d)                'insert extra line 
      if subsc<10                 'for numbers less than 10
        fds.str(string(" "))      'add a space to align numbers
      fds.dec(subsc)              'display P_Val variable
      fds.str(string("   "))      'print trailing spaces
      fds.bin(P_Val[subsc], bits) 'print variable as bits
      fds.str(string("   "))      'spaces
      fds.dec(P_Val[subsc])       'print variable as decimal
      fds.str(string("   "))      'spaces
      fds.tx($d)                  'new line

    waitcnt(clkfreq/60+cnt)'flicker free wait
    x:=x+1       'increment counter. This routine clears up the screen    )
    if x>100     'decision point     by erasing extraneous lines of       )
      fds.tx(16) 'clear screen       bottom of PST display /100 loops     )
      x:=0       'reset counter      needed only once in most cases.      )

'==================  End of Cog 0, the PST display Cog ========================
DAT
          org     0                    'start address
timer60   mov     dira,     Set_dira   'set pin directions
          mov     outa,     Set_outa   'set pin I/O
          mov     mem,      par        'read par address 
          mov     mem1,     mem        '5th mem location
          add     mem1,     #16        'to first free location
          mov     mem2,     mem        '6th memory location
          add     mem2,     #20        'to second free location

fillp     mov     delay60,  long_P     'load 1/60 sec counter
:loop     djnz    delay60,  :loop      'delay 1/60                                                  '
          call    #signal              'call servo pulse subroutine
          jmp     #fillp               'do again
'
signal    rdlong  temp,     mem         'read pot0
          wrlong  temp,     mem1        'write to loc 5 for check on PST
          shl     temp,     #3          'multiply by 8
          add     temp,     short_p     'add correction for pulse
          wrlong  temp,     mem2        'write to loc 6 for check on PST
          or      outa,     mask0       'signal high on pin 0
first     djnz    temp,     #first      'variable short pulse delay
          andn    outa,     mask0       'signal low on pin 0
signal_ret   Ret                        'return from subroutine
               
long_P     long   38_0000        'approx 1/60 seconds
short_P    long   14200          'short pulse correction

set_dira   long   %00000000_00000000_00000000_00000001
set_outa   long   %00000000_00000000_00000000_00000001
mask0      long   %00000000_00000000_00000000_00000001

mem        res    1    'pot0 address
mem1       res    1    'pot0 as read
mem2       res    1    'adjusted pulse width
temp       res    1    'pot0 as used
delay60    res    1    'long delay decrementing holder
Fit