{{-------------------------------------------------------------------------------
B00k_PASM_Follow_PB.SPIN
Function of program
This program demos an LED controlled by a push button.
Program turns an LED on if and only if a PB is down.
Input  on line 1, PB switch 0
Output on line 0, to an LED

Program         Book_PASM_Follow_PB.SPIN
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Jan 20 2012

Revisions:      Report errors to harprit.sandhu@gmail.com

-------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
VAR
  long P_Val[4]       'we dont need 4 variables.
  
OBJ
  fds : "FullDuplexSerial"

PUB Go|x
fds.start(31,30,0,115200)       'start console at 115200 for debug output                                            'call main  
Cognew(@Button,   @P_Val)       'start Cog1
waitcnt(clkfreq/4 +cnt)         'wait for all to start up
dira[0]~~                       'set pin 0 to output to LED
dira[1]~                        'set pin 1 to input from Pin
fds.tx($1)                            'clear, to 0,0
fds.str(string("Display results"))    'header
fds.tx($d)                      'new line
  repeat                        'loop to display data
    fds.tx(1)                   'home
    if P_Val[0]==1              'print to the PST in binary
      fds.str(string("Button down "))
      fds.tx($d)                'new line
      fds.str(string("LED is ON   "))
    else                        'print value as decimal value  
      fds.str(string("button up   "))
      fds.tx($d)                'new line
      fds.str(string("LED is OFF  "))
    fds.tx($d)                  'new line
    waitcnt(clkfreq/60+cnt)     'flicker free wait
    x:=x+1             'increment counter      This routine clears up screen
    if x>100           'decision point         by erasing extra lines of bot
      fds.tx(16)       'clear screen           of PST display every 100 loops
      x:=0             'reset counter          Only really needed once.
 
DAT            org       0
Button         mov       dira,     set_dira    'set dira
               mov       outa,     set_outa    'set outa
               mov       mem,      par         'Read Par pointer
          
Redo           mov       temp,     ina         'read ina into temp
               andn      temp,     mask1 wz    'mask to look at pin 1
if_z           jmp       #redo                 'do it again if 0
               wrlong    up,       mem         'write new state to hub memory
               call      #pause_Sub
               andn      outa,     mask0
               
again          mov       temp,     ina         'read ina into temp
               andn      temp,     mask1 wz    'mask to look at pin 1
if_nz          jmp       #again                'do it again if 1
               wrlong    dn,       mem         'write new state to hub memory
               call      #pause_Sub
               or        outa,     mask0
               jmp       #redo                 'do it over again
'===============================
pause_Sub      mov       temp1,     delay
:loop          djnz      temp1,     #:Loop
pause_Sub_ret   ret
'===============================
'constants and variables
delay     long    2_000_000   '1/40 sec
set_dira  long    %00000000_00000000_00000000_00000001
set_outa  long    %00000000_00000000_00000000_00000000 
mask0     long    %00000000_00000000_00000000_00000001
mask1     long    %11111111_11111111_11111111_11111101 
up        long    0
dn        long    1

mem       res     1
temp      res     1
temp1     res     1
Fit