
{{-----------------------------------------------------------------------------
Book_PASM_WaveLength.SPIN
Function of program
The SPIN Cog displays the information being generated.
The next Cog generates a square wave based on a seeded value that you can change.
A third Cog is used to read the wavelength generated by the previous Cog.

Notes:  We are not converting the wavelength read into a frequency.
It would be 80_000_000 divided by the wavelength read.

A trap for P_VAL[4] is provided in each PASM cog to inhibit starting these cogs.

You can see generated frequency on pin 0

Program         Book_PASM_WaveLength.SPIN 
Book            PASM for beginners, Propeller 102
Section         Inputs
Programmer      Harprit Sandhu   
Date            Jan 20 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

var
  long  P_Val[4]               'variables we look at on the PST

OBJ
  fds : "FullDuplexSerial"       'for PST display

PUB wavelengths|x                'x becomes a local variable
fds.start(31,30,0,115200)        'start console at 115200 for debug output
waitcnt(clkfreq/100+cnt)         'wait for startup  
cognew(@prog_gen, @P_Val)        'start new Cog to generate freq                                                             
waitcnt(clkfreq/100+cnt)         'wait for startup 
cognew(@prog_red, @P_Val)        'start new Cog to read freq
waitcnt(clkfreq/60+cnt)          'wait for startup
  repeat                         'loop
    fds.tx($1)                   'home to 0,0
    fds.str(string("Frequency delay, 2 CNT counts"))
    fds.tx($0d)                  'new line
    fds.str(string("and wave length in clk cycles"))
    fds.tx($0d)                  'new line       
    fds.tx($0d)                  'new line 
    fds.dec(P_Val[0])            'print value as decimal
    fds.tx($0d)                  'new line 
    fds.dec(P_Val[1])            'print value as decimal 
    fds.tx(" ")                  'space 
    fds.tx($0d)                  'new line
    fds.dec(P_Val[2])            'print value as decimal 
    fds.tx(" ")                  'new line
    fds.tx($0d)                  'space
    fds.dec(P_Val[3])            'print value as decimal 
    fds.tx(" ")                  'new line

    waitcnt(clkfreq/60+cnt)       'flicker free wait
    x:=x+1              'increment counter. (This routine clears up the screen    )
    if x>100            'decision point     (by erasing extraneous lines of       )
      fds.tx(16)        'clear screen       (bottom of PST display every 100 loops)
      x:=0              'reset counter      (needed only once in most cases                                                 
    P_Val[4]:=1

dat       org     0                    'program generates a sq wave
prog_gen  mov     mem0,    Par          ')                              
          add     mem0,    #16          ')                              
          mov     temp,   #0           ')trap waits for PVal[4] to = 1
          wrlong  temp,   mem0          ')                              
:loop9    rdlong  temp,   mem0  wz      ')                              
if_z      jmp     #:loop9              ')                              

          mov      dira,     set_dira   'to set directions
          mov      mem,      par        'read par and store in mem
          wrlong   del_cnt,  mem        'store delay value used for wave

out1      or       outa,     pin        'pin 0 high
          call     #clk_delay           'call delay
          andn     outa,     pin        'pin 0 low
          call     #clk_delay           'call delay 
          jmp      #out1                'do again
'================================== 
clk_delay mov      time,     del_cnt    'load del_cnt into time
out5      sub      time,     #1 wz      'sub 1 from time and set flag if 0
if_nz     jmp      #out5                'if flag not 0 go back to take5
clk_delay_ret      ret                  'return for delay subroutine 
'===================================
del_cnt    long   10_024     'time of delay (constant for now)
set_dira   long   %00000000_00000000_00000000_00000001
pin        long   %00000000_00000000_00000000_00000001
time   res   1
temp   res   1
mem0   res   1
fit
'=================================
          org     0 
prog_red  mov     mem,    Par        ')
          add     mem,    #16        ')
          mov     temp2,  #0         ')trap waits for PVal[4] to = 1
          wrlong  temp2,  mem        ')
:loop9    rdlong  temp2,  mem  wz    ')
if_z      jmp     #:loop9            ')

          mov     mem,    par        'read
          mov     dira,   set_dira   'to set directions                  
          mov     mem1,   mem
          add     mem1,   #4         'set up mem1
          mov     mem2,   mem
          add     mem2,   #8         'set up mem2  
          mov     mem3,   mem
          add     mem3,   #12       'set up mem2
          
out2      mov     temp4,   ina        'look at INA register
          andn    temp4,   mask0 wz   'look at only pin 0
if_z      jmp     #out2              'stay here while it is low
          mov     temp2,  cnt        'look at INA register
          wrlong  temp2,  mem1       'store value
          
out3      mov     temp4,   ina        'look at INA register
          andn    temp4,   mask0 wz   'look at only pin 0
if_nz     jmp     #out3              'stay here while high
            
out4      mov     temp4,   ina        'look at INA register
          andn    temp4,   mask0 wz   'look at only pin 0
if_z      jmp     #out4              'stay here while low
          mov     temp3,  cnt        'look at INA register
          wrlong  temp3,  mem2       'store value
          sub     temp3,  temp2      'figure time interval
          wrlong  temp3,  mem3       'write it to mem3
          jmp     #out2               'do it again
'===================
mask0          long %11111111_11111111_11111111_11111110
set_outa       long %00000000_00000000_00000000_00000001

temp2          res       1
temp3          res       1
temp4          res       1
mem            res       1
mem1           res       1
mem2           res       1 
mem3           res       1
Fit
