{{-------------------------------------------------------------------------------
PASM_WaveLength_A.SPIN
Function of program
Count waves for 1/20 seconds time
Th SPIN cog displays the information being generated.
The next cog generates a square wave based on a seeded value that you can change.
A third cog is used to count the waves.

Notes:
We are counting the number of waves generated in 1/20 seconds.

I had to wait between starting the PASM programs to make sure they started
before I addressed the next cog. There does not seem to be a recognized
protocol for a sequence for starting these cogs.
A short delay works.  Sometimes you don't need a delay for non PASM code.

Program         PASM_WaveLength_A.SPIN 
Book            PASM for beginners, Propeller 102
Section         Intputs
Programmer      Harprit Sandhu   
Date            Jan 20 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

--------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

var
  long  P_Val[8]               'variables we look at on the PST
  
OBJ
  fds : "FullDuplexSerial"     'for PST display
 
PUB wavelengths|x               'x becomes a local variable
fds.start(31,30,0,115200)       'start console at 115200 for debug output
waitcnt(clkfreq/100+cnt)        'wait for startup  
cognew(@gen_Wav, @P_Val)        'start new cog to generate freq
cognew(@Count_Wav, @P_Val)      'start new cog to read wave count
  repeat                        'loop
    fds.tx($1)                  'home to 0,0
    fds.str(string("Data display 2 counter readings "))
    fds.tx($0d)                 'new line
    fds.str(string("waves read in time period"))
    fds.tx($0d)                'new line               
    fds.tx($0d)                'new line
    fds.dec(P_Val[0])          'print value as decimal
    fds.tx($0d)                'new line
    fds.dec(P_Val[1])          'print value as decimal 
    fds.str(string("    "))    'spaces
    fds.tx($0d)                'new line
    fds.dec(P_Val[2])          'print value as decimal 
    fds.str(string("    "))    'spaces
    fds.tx($0d)                'new line
    
    waitcnt(clkfreq/60+cnt)       'flicker free wait
    x:=x+1        'increment counter. (This routine clears up the screen)
    if x>100      'decision point     (by erasing extraneous lines of)
      fds.tx(16)  'clear screen       (bottom of PST display every 100 loops)
      x:=0        'reset counter      (needed only once in most cases.)                                             
    P_Val[4]:=1
    
dat       org     0                    'program generates a sq wave
gen_Wav   mov     mem0,    Par         ')                              
          add     mem0,    #16         ')                              
          mov     temp,   #0           ')trap waits for PVal[4] to = 1
          wrlong  temp,   mem0         ')                              
:loop9    rdlong  temp,   mem0  wz     ')                              
if_z      jmp     #:loop9              ')

          mov     dira,     set_dira    'to set directions
          mov     mem0,      par         'read par and store in mem
          wrlong  del_cnt,  mem0         'store delay value used for wave

out1      or      outa,     pin         'pin 0 high
          call    #clk_delay            'call delay
          andn    outa,     pin         'pin 0 low
          call    #clk_delay            'call delay
          jmp     #out1                 'do again
'=================================
clk_delay mov     time,     del_cnt     'load del_cnt into time
out2      sub     time,     #1 wz       'sub 1 from time and set flag if 0
if_nz     jmp     #out2                 'if flag not 0 go back 
clk_delay_ret     ret                   'return 
'=================================
del_cnt    long   1_000        'time of delay (freq delay)
set_dira   long   %00000000_00000000_00000000_00000001
pin        long   %00000000_00000000_00000000_00000001 
'=================================
temp      res     1  
mem0      res     1 
time      res     1
fit
'========================================================================
'========================================================================
          org     0                    'program generates a sq wave
Count_Wav mov     mem5,    Par         ')                              
          add     mem5,    #16         ')                              
          mov     temp2,   #0          ')trap waits for PVal[4] to = 1
          wrlong  temp2,   mem5        ')                              
:loop9    rdlong  temp2,   mem5  wz    ')                              
if_z      jmp     #:loop9              ')

          mov     dira,     set_dira1   'to set DIRA directions  
          mov     mem5,     par         'read par into mem5                                                                                        
          mov     mem6,     mem5         
          add     mem6,     #4          'set up mem6
          mov     mem7,     mem5
          add     mem7,     #8          'set up mem7 

loop       mov    nmbr,    cnt          'read initial sys coounter                                               
           add    nmbr,    del_long     'add active time to it
           mov    wave,     #0          'reset set waves counted
           
loop2     call    #wait_lo          'is pin 0 lo
          call    #wait_hi          'is pin 0 hi
          mov     nmbr2,    cnt     'read sys counter, (lo to hi tranistion)  
          add     wave,     #1      'add 1 to waves counted
                               
          mov     dis5,     nmbr    'counter1
          mov     dis6,     nmbr2   'counter2  
          mov     dis7,     wave    'waves counted so far 
          
          sub     nmbr2,    nmbr wc 'get difference                                                                                                       
if_b      jmp     #loop2            'if more time available do next wave
          wrlong  dis5,     mem5    'write first count
          wrlong  dis6,     mem6    'write second count
          wrlong  dis7,     mem7    'write waves counted         
          jmp     #loop
'============
wait_lo   mov     temp2, ina        'read inputs
          andn    temp2, mask0  wz  'read bit 0
if_nz     jmp     #wait_lo          'wait if not 0
wait_lo_ret       ret               'do it again
'============
wait_hi   mov     temp2, ina        'read inputs     
          andn    temp2, mask0  wz  'read bit 0      
if_z      jmp     #wait_hi          'wait if 0   
wait_hi_ret       ret               'do it again    
'============                             
mask0     long    %11111111_11111111_11111111_11111110
set_dira1 long    %00000000_00000000_00000000_00000001
del_long  long    4_000_000     '1/20 sec

nmbr      res     1
nmbr2     res     1
wave      res     1
temp2     res     1
dis5      res     1
dis6      res     1
dis7      res     1
mem       res     1
mem5      res     1
mem6      res     1
mem7      res     1
Fit
