{{------------------------------------------------------------------------------
Book_PASM_MIN_PWM.SPIN
Function of program
Program creates minimal PWM signal on pin 0
Connect it to an LED to see dimming.
The response of the LED is very interesting.
Not linear!  Easy to turn on and saturate!

Program         Book_PASM_MIN_PWM.SPIN
Book            PASM for beginners, Propeller 102
Section         Inputs
Programmer      Harprit Sandhu   
Date            Jan 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

-------------------------------------------------------------------------------}}
CON
  _clkmode = XTAL1 + PLL16x
  _xinfreq = 5_000_000

VAR
  long  P_Val[4]   
   
OBJ 
  fds : "FullDuplexSerial"      'for communications with PST
  r4p : "read4pots"             'reads pots 
  
PUB Main|x 
fds.start(31,30,0,115200)       'start console at 115200 for debug output  
r4p.Read4Pots(@P_Val)           'start pot reading loop in read4pots object, start first        
cognew(@PWM, @P_Val)           'Cog in PASM for freq gen  
fds.tx(1)                       'clear screen
repeat         
  fds.tx(1)                   'home
  fds.dec(P_Val[0])           'print pot0 value
  fds.tx(" ")                 'space
  fds.tx($d)                  'next line

  waitcnt(clkfreq/60+cnt)     'flicker free wait
  x:=x+1                      'increment counter. This routine clears up screen
  if x>100                    'decision point     by erasing extra lines of bot
    fds.tx(16)                'clear screen       of PST display every 10 loops
    x:=0                      'reset counter
'--------------------------------------------------------   
DAT       org 0
PWM       mov      dira,      set_dira_M  'set dira register
          mov      ctra,      counter     'set counter, mode, I/O etc
          mov      mem,       par         'read par
          mov      frqa,      #1          'set frqa
          mov      period,    cnt         'read counter
          add      period,    #20         'add to prevent overflow
:loop     mov      P_width,   #0          'reset puls width min
          rdlong   pot0,      mem         'read potentiometer 0
          subs     P_width,   pot0        'set pulse with Pot0
          subs     P_width,   #60         'add 60 for min compensation
          mov      phsa,      verb        'set phase
          add      period,    wav1        'add to counter for next wave end
          waitcnt  period,    wav1        'wait for end of wavbe
          jmp      #:Loop                 'loop back

'Constants and variables                                     
set_dira_M        long    %00000000_00000000_00000000_00000001
counter           long    %00010000_00000000_00000000_00000000
wav1              long    2200
P_width           long    0 
period            res     1
pot0              res     1
mem               res     1
fit
