{{-------------------------------------------------------------------------------
Book_PASM_LM34.SPIN
Function of program
Implements the use of an LM34 temperature sensor
in PASM. Data is written to the HUB memory.
Views data on PST
See book for details and operation scheme

Connections are as follows
MCP3208 with LM34 on line 1

Program         Book_PASM_LM34.SPIN                       
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Feb 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

-------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
VAR
  long POT_RES
  
OBJ 
  fds : "FullDuplexSerial"
  
PUB Main |x, P_VAL          'P_VAL is a local variable here
fds.start(31,30,0,115200)   'start console at 115200 for debug output
cognew(@Read, @P_Val)      'start new cog at "generate" read var to P_Val
repeat                     'endless loop to display data
  POT_RES:=P_VAL           'reads in Hub data
  fds.tx($1)               'home to 0,0
  fds.str(string("Raw data = "))
  fds.bin(P_val,12)        'print to the PST in binary 
  fds.tx($d)               'new line 
  fds.str(string("           "))
  fds.dec(P_val)           'print value as decimal value                                 
  fds.tx($d)               'new line 
  fds.tx($d)               'new line 
  fds.str(string("Temperature"))
  fds.tx($d)                    'new line 
  fds.dec(100*Pot_res/1365)     'spaces                                                    
  fds.str(string(" degrees F")) 
  fds.tx($d)                    'new line                                                 
  fds.str(string("  "))         'spaces
 
  waitcnt(clkfreq/60+cnt)      'wait 1/60.
  x:=x+1             'increment counter. (This routine clears up the screen    )
  if x>10            'decision point     (by erasing extraneous lines of       )
    fds.tx(16)       'clear screen       (bottom of PST display every 10 loops )
    x:=0             'reset counter      (Needed only once in most cases.      )
'==================================
DAT       org     0                  'sets the starting point in Cog
Read      mov     dira,    set_dira  'sets direction of the prop pins
          or      outa ,   chs_Bit   'Makes Chip select high
          andn    outa,    chs_Bit   'makes chip select low 
          andn    outa ,   clk_bit   'ANDN it with the Clock Bi to make low
          or      outa ,   din_Bit   'Makes the Din high
          call    #Tog_clk           'clk hi-lo to read data 
          or      outa ,   din_Bit   'Makes the Din high 
          call    #Tog_Clk           'toggle clock line hi-low to read data                 
          andn    outa ,   din_Bit   'makes Din low  000 for line 0
          call    #Tog_Clk           'toggle clock line hi-low to read data            
          andn    outa ,   din_Bit   'makes Din low  000 for line 0
          call    #Tog_Clk           'toggle clock line hi-low to read data           
          andn    outa ,   din_Bit   'makes Din low  000 for line 0
          call    #Tog_Clk           'toggle clock line hi-low to read data                 
          andn    outa ,   din_Bit   'makes Din low
          call    #Tog_Clk           'toggle clock line hi-low to read data
          call    #Tog_Clk           'toggle clock line hi-low to read data                        
          mov     dat_red,  #0       'Clear register we will read data into             
          mov     count,    #12      'Counter for number of bits we will read
read_bit  mov     temp,    ina       'read in what is in all the input lines
          andn    temp,    mask26 wz 'mask off except Dout line. Set Z flag    
          shl     Dat_red,  #1       'shift register left 1 bit for next bit
if_nz     add     Dat_red,  #1       'if value + add 1 to data register    
          call    #Tog_Clk           'toggle clock get next bit ready in Dout
          sub     count,    #1    wz 'decr the "bits read" counter. Set Z flag
if_nz     jmp     #read_bit          'go up do it again if counter not yet 0
          wrlong  dat_red,  mem      'write it in PAR to share it as P.Val
          mov     mem,      par      'get address of mem 
          or      outa ,    chs_Bit  'Makes Chip select high
          jmp     #read              'go back to do it all again
'===============================================================================              
Tog_Clk   or      outa,   clk_bit     'make clock bit high
          andn    outa,   clk_bit     'make clock bit low
Tog_Clk_ret       ret                 'return from this subroutine
'======================           
Set_dira  long   %00001011_00000000_00000000_00000000   'Set dira register                                 
Chs_Bit   long   %00000001_00000000_00000000_00000000   'Chip select bit     24
Din_Bit   long   %00000010_00000000_00000000_00000000   'Data in bit         25
Dout_Bit  long   %00000100_00000000_00000000_00000000   'Data out bit        26
Clk_Bit   long   %00001000_00000000_00000000_00000000   'Clock bit           27
mask26    long   %11111011_11111111_11111111_11111111   'Mask to read Dout bit          

temp      res    1       'temporary storage variable, misc
count     res    1       'temporary storage variable, read bit counter
Dat_Red   res    1       'temporary storage variable, data being read
mem       res    1       'memory location for PAR variable
fit
