{{
022 PASM Read4Pots.SPIN
Program to read 4 pots
Installs in separate Cog.
Reads resistances to 12 bits appears in P_Val global variables
15 Dec 2011
By Harprit Sandhu
}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  bits=12
  pots=4
  
VAR
  long P_Val[4]      'there will be 4 variables starting at P_Val[0]
  long  pot_sub      'subscript variable for identifying potentiometers
  
OBJ 
  fds : "FullDuplexSerial" 
  
PUB Main|clear                            'P_VAL is a local variable here
fds.start(31,30,0,115200)           'start console at 115200 for debug output 
waitcnt(clkfreq/60 +cnt) 
Read4Pots (@P_Val)                  'start new Cog at "generate", read var to P_Val 
dira[0..1]~~                        'set pin directions to outputs
fds.tx(16)                          'clear screen
  repeat                            'endless loop 
    pot_sub:=0                      'reset id of pot to be printed
    fds.tx($1)                      'home to 0,0
    repeat pots                     'loop to display data
      fds.bin(P_val[pot_sub], bits) 'print to the PST in binary 
      fds.str(string(" "))          'spaces to erase old data overflow 
      fds.dec(P_val[pot_sub])       'print value as decimal value 
      fds.str(string(" "))          'spaces to erase old data overflow  
      fds.dec(pot_sub)              'print value as decimal value 
      fds.str(string(" "))          'spaces to erase old data overflow
      fds.tx($d)                    'new line
      pot_sub:=(pot_sub)+1          'increment display counter
    waitcnt(clkfreq/60+cnt)         'flicker free wait
    clear:=clear+1  'increment counter. (This routine clears up screen )
    if clear>10     'decision point     (by erasing extra lines of bot )
      fds.tx(16)    'clear screen       (of PST display every 10 loops )
      clear:=0      'reset counter



PUB Read4Pots(value)
  cognew(@Read_Pots, value)
  
DAT           org       0             'sets the starting point in Cog
Read_Pots     'first set up all the counters and addresses needed
              mov   pots_read, #4    'number of pots to read 
              mov   pot_id,    #0    'first pot read is pot 0
              mov   bits_read, #12   'bit resolution of 3208 IC, can be less
              mov   mem,       par   'get address of mem for PAR
              mov   dira,      set_dira  'sets direction of the prop pins 
                                     
Next_pot      or    outa ,   chs_Bit 'makes Chip select high
              andn  outa,    chs_Bit 'makes chip select low 
              andn  outa ,   clk_bit 'ANDN it with the Clock Bit to make low
              or    outa ,   din_Bit 'makes the Din high
              call  #Tog_clk         'toggle clock line hi-low to read data
              or    outa ,   din_Bit 'makes the Din high 
              call  #Tog_Clk         'toggle clock line hi-low to read data
              mov   temp2,     pot_id    'we will read three bits from this pot
              call  #get_third_bit       'get bit
       if_nz  jmp   #its_one             'jump to set it as needed
              jmp   #its_zero            'jump to set it as needed
its_one       call  #Set_next_bit1       'setting bit subroutine call
              jmp   #continue1           'go to next bit
its_zero      call  #Set_next_bit0       'setting bit subroutine call
continue1     call  #get_second_bit      'get bit
       if_nz  jmp   #its_one1            'jump to set it as needed
              jmp   #its_zero1           'jump to set it as needed
its_one1      call  #Set_next_bit1       'setting bit subroutine call
              jmp   #continue2           'go to next bit
its_zero1     call  #Set_next_bit0       'setting bit subroutine call
continue2     call  #get_first_bit       'get bit
       if_nz  jmp   #its_one2            'jump to set it as needed
              jmp   #its_zero2           'jump to set it as needed
its_one2      call  #Set_next_bit1       'setting bit subroutine call
              jmp   #continue3           'go to next bit
its_zero2     call  #Set_next_bit0       'setting bit subroutine call

continue3     andn  outa ,   din_Bit     'makes Din low 
              call  #Tog_Clk             'toggle clock line hi-low to read data
              call  #Tog_Clk             'toggle clock line hi-low to read data                    
              mov   Data_read,  #0       'clear register read data into 
              mov   bit_count, bits_read 'counter for  bits we will read
read_bit      mov   temp,     ina        'read all the input lines
              andn  temp,     mask26 wz  'mask off except Dout line. Set Z
              shl   Data_read,  #1       'shift reg left 1 bit for next bit
        if_nz add   Data_read,  #1       'if value + add 1 to data register    
              call  #Tog_Clk             'toggle clock get  bit ready in Dout
              sub   bit_count, #1 wz     'decr the "bits read" counter. Set Z 
        if_nz jmp   #read_bit            'go back if counter not yet 0 
              wrlong Data_read,  mem     'write it in PAR to share it as P.Val
              add   mem,  #4             'add 4 to hub address for next long
              add   pot_id, #1           'so we can look at next pot
              mov   temp2, pot_id        'recall what pot we are reading
              sub   temp2, pots_Read wz  'check if it is how many we want to read
      if_nz   jmp  #Next_pot        'if it is not 0 go up and read next pot
              jmp  #Read_pots       'go back beginning and do all pots again

'subroutines used
Set_next_bit0  andn    outa, din_Bit   'makes Din low in 000 for line
              call #Tog_Clk         'toggle clock line hi-low to read data 
Set_next_bit0_ret  ret              'return from this subroutine

Set_next_bit1 or   outa ,    din_Bit   'makes Din high in 000 for line
              call #Tog_Clk        'toggle clock line hi-low to read data 
Set_next_bit1_ret  ret             'return from this subroutine

Tog_Clk       nop                  'nop to settle signals
              or   outa,  clk_bit  'make clock bit high
              nop                  'nop to settle signals
              andn outa,  clk_bit  'make clock bit low
              nop                  'nop to settle signals
Tog_Clk_ret   ret                  'return from this subroutine 

Get_first_bit  mov temp2, pot_id   'get current pot number
               andntemp2, mask0 wz 'get last bit
Get_first_bit_ret  ret             'return

Get_second_bit mov temp2, pot_id   'get current pot number
               shr temp2, #1       'shift right 1 bit to get second bit
               andntemp2, mask0 wz 'return 
Get_second_bit_ret ret            

Get_third_bit  mov temp2, pot_id   'get current pot number
               shr temp2, #2       'shift right 2 bits to get third bit
               andntemp2, mask0 wz 'return 
Get_third_bit_ret  ret  
 
Set_dira  long  %00001011_00000000_00000000_00001111 'Set dira register                            
Chs_Bit   long  %00000001_00000000_00000000_00000000 'Chip select bit     24
Din_Bit   long  %00000010_00000000_00000000_00000000 'Data in bit         25
Dout_Bit  long  %00000100_00000000_00000000_00000000 'Data out bit        26
Clk_Bit   long  %00001000_00000000_00000000_00000000 'Clock bit           27
mask26    long  %11111011_11111111_11111111_11111111 'Mask to read Dout bit 26
mask0     long  %11111111_11111111_11111111_11111110 'Mask to read bit 0                                                                                         

mem         res   1  'Par location
temp        res   1  'temporary storage variable, misc
temp2       res   1  'temporary storage variable, misc
bit_count   res   1  'temporary storage variable, read bit counter
Data_Read   res   1  'temporary storage variable, data being read 
pot_id      res   1  'current pot id number
pots_read   res   1  'total number of pots to be read.
bits_Read   res   1  'resolution of data read, 12 bits max
Fit
