{{------------------------------------------------------------------------------
                                                                                      
Book_PASM_One_MilliSec.SPIN
Function of program
Create a one Millisecond delay

Notes:
Cog0 displays info on the PST
Cog1 creates the 1 microsec signal.
Cog2 measures the exact time between the time signals
Cog1 timing is then adjusted manully to get it right on 80_000 cycles
as close as possible, which corresponds to 0.001 seconds.

Signal is on Pin 7

Program         Book_PASM_One_MilliSec.SPIN
Book            PASM for beginners, Propeller 102
Section         Inputs
Programmer      Harprit Sandhu
Date            Jan 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

--------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

OBJ 
  fds : "FullDuplexSerial"         'for communications with PST
  
VAR
  long P_Val[8]           'we need only one variable
   
PUB Main|x,y 
fds.start(31,30,0,115200)    'start console at 115200 for debug output
Cognew(@Milli_S,   @P_Val)   'start PASM program
Cognew(@Reader,    @P_Val)    'start width reader
waitcnt(clkfreq/40 +cnt)     'wait for cogs to startup 
fds.tx(16)                   'clear screen
'dira[0]~
'dira[1]~~
repeat     
  fds.tx($1)                 'home  
  fds.str(string("Mem0 is not used ="))   'spaces
  fds.dec(||P_Val[0])        'print value
  fds.str(string("   "))   'spaces
  fds.tx($d)               'new line 
  fds.str(string("Short delay  ="))   'spaces
  fds.dec(||P_Val[1])        'print value
  fds.str(string("   "))   'spaces
  fds.tx($d)               'new line 
  fds.str(string("Long delay   ="))   'spaces
  fds.dec(||P_Val[2])        'print value
  fds.str(string("   "))   'spaces
  fds.tx($d)               'new line 
  fds.str(string("Counter 1    ="))   'spaces
  fds.dec(||P_Val[3])        'print value
  fds.str(string("   "))   'spaces
  fds.tx($d)               'new line  
  fds.str(string("Counter 2    ="))   'spaces
  fds.dec(||P_Val[4])        'print value
  fds.str(string("   "))   'spaces
  fds.tx($d)               'new line
  fds.str(string("Diff is      ="))   'spaces
  fds.dec(||P_Val[5])        'print value
  fds.str(string("   "))   'spaces
  fds.tx($d)               'new line
    
  waitcnt(clkfreq/60+cnt) 'flicker free wait
  x:=x+1                  'increment counter. This routine clears up screen
  if x>100                'decision point     by erasing extra lines of bot
    fds.tx(16)            'clear screen       of PST display every 100 loops
    x:=0                  'reset counter

'--------------------------------------------------------------------------------
DAT       org     0
Milli_S   mov     dira,     dira_set    'sets direction of the prop pins
          mov     mem,      par         'set mem from PAR
          mov     mem1,     mem         '
          add     mem1,     #4          'set mem1
          mov     mem2,     mem         '
          add     mem2,     #8          'set mem2
                                         
          wrlong delay,     mem1        '
          wrlong delay2,    mem2
                                        '
again     or      outa,     mask7       'set line 1 hi
          mov     temp,     delay  
:loop     djnz    temp,     #:loop      ' 
                                        '
          andn    outa,     mask7       'set line 1 low 
          mov     temp,     delay2 
:loop2    djnz    temp,     #:loop2     ' 
          jmp     #again               'if still high look again
                          
dira_set  long    %00000000_00000000_00000000_11110000 'Set dira register                                                                                                                                                                         
mask7     long    %00000000_00000000_00000000_10000000
delay     long    500
delay2    long    19_507
mem       res 1
mem1      res 1
mem2      res 1
temp      res 1
fit                 
'==============================
          org     0                                                                          
Reader    mov     mem0,     par          'set mem from PAR  
          mov     mem3,     mem0        '
          add     mem3,     #12         'set mem3
          mov     mem4,     mem0        '
          add     mem4,     #16         'set mem4 
          mov     mem5,     mem0        '
          add     mem5,     #20         'set mem5
          
loop_3    mov     inbits,   ina          'read input register
          andn    inbits,   mask7R wz    'look at pin 1
    if_nz jmp     #loop_3                'if lo stay here
          mov     first,    cnt          'just went high
          wrlong  first,    mem3         'write counter to mem3

loop_4    mov     inbits,   ina
          andn    inbits,   mask7R wz    'look at pin 1
if_z      jmp     #loop_4                'if lo stay here

loop_5    mov     inbits,   ina
          andn    inbits,   mask7R wz    'look at pin 1
if_nz     jmp     #loop_5                'if hi stay here
          mov     second,   cnt          'just went hi
          wrlong  second,   mem4         'write counter to mem4 

          sub     second,   first        'calculate difference
          wrlong  second,   mem5         'write counter to mem5
          jmp     #loop_3
fit
'=======================
dira_set1 long    %00000000_00000000_00000000_11110000 'Set dira register                                                                                                               
mask7R    long    %11111111_11111111_11111111_01111111
'=======================
'variables
first     res     1
second    res     1
inbits    res     1
mem0      res     1         'memory locations re PAR                                                  
mem3      res     1                                                                  
mem4      res     1
mem5      res     1  
fit
