{{-------------------------------------------------------------------------------
Book_PASM_Thermostat.SPIN
Function of program
Program to read 1 pot to set thermostat and read
the LM34 to determine temperature to be controlled
Reads variables to 12 bits .

Notes:  Program reads 3 pots and LM34 to 12 bits and stores
the information in the first 4 Hub/Par locations. The number of
pots and the bit resolution can be specified on the first few lines
of code. It then converts the LM34 reading to a temperature and turns
on bit0 for control purposes. Bit6 is turned as an LED for feedback.

Program         Book_PASM_Thermostat.SPIN
Book            PASM for beginners, Propeller 102
Section         Projects
Programmer      Harprit Sandhu   
Date            Jan 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

--------------------------------------------------------------------------------}}

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  bits=12          'read MCP3208 to this resolution
  pots=4           'number of pots to read.
                     
VAR  
  long P_Val[Pots]                 'five variables

OBJ 
  fds : "FullDuplexSerial"         'for communications with PST
  r4p : "read4pots"                'potentiometers
  
PUB Main |x,pot_sub                'x, pot_sub are local variables here
fds.start(31,30,0,115200)          'start console at 115200 for debug output 
r4p.read4pots (@P_Val)             'start new cog "generate", read var to P_Val
waitcnt(clkfreq/60 +cnt)           'allow things to settle down, initialize
dira[0]~~                          'set lines to outputs 
dira[6]~~                          'set lines to outputs
  repeat                           'endless loop
    pot_sub:=0                     'set subscript value
    fds.tx($1)                     'home to 0,0
    fds.str(string("Temperature   "))     'print to display
    fds.dec(P_val[0]*300/4090)            'print value as decimal value
    repeat 2                      'two spaces
      fds.tx(" ")                 'print 2 spaces to erase old data overflow
    fds.tx($d)                    'new line
    pot_sub:=1                    'set subscript value
    fds.str(string("Set point is  "))     'print to display
    fds.dec(P_val[3]*300/4090)            'print value, range is 0-300
    repeat 2                      'print 2 spaces to erase old data overflow
      fds.tx(" ")                 'space to erase old data overflow
    fds.tx($d)                    'new line 
    fds.tx($d)                    'new line 

    fds.str(string("Power is      "))      'print to display
    if P_val[0]<P_val[3]                   'check state of line 5
      fds.str(string("ON"))        'print ON
      fds.tx(" ")                  'print a space
      outa[0]~~                    'set lines 0 high
    else                           'else
      fds.str(string("OFF"))       'print OFF
      outa[0]~                     'set lines 6 low 
    fds.tx($d)                     'new line 

    fds.str(string("LED is        "))     'print to display
    if P_val[0]<P_val[3]                   'check state of line 5
      fds.str(string("ON"))        'print ON
      fds.tx(" ")                  'print a space
      outa[6]~~                    'set lines 6 high
    else                           'else
      fds.str(string("OFF"))       'print OFF
      outa[6]~                     'set lines 6 low

    waitcnt(clkfreq/60+cnt)        'flicker free wait
    x:=x+1                 'increment counter. This routine clears up screen
    if x>100               'decision point     by erasing extra lines of bottom
      fds.tx(16)           'clear screen       of PST display every 100 loops
      x:=0                 'reset counter
Fit
