{{-------------------------------------------------------------------------------
PASM_WaveLength1.SPIN
Function of program
Count waves for 1/20 seconds time
Th SPIN Cog displays the information being generated.
The 2nd Cog generates a square wave based on a seeded value that you can change.
A third Cog is used to count the waves.
No external connections are needed if there is no signal to look at
Signal can be monitored on line 1

Notes:
We are counting the number of waves generated in 1/20 seconds.
Trap added to inhibit PASM cogs till the SPIN Cog is fully functional.
See discussion in book.

Program         PASM_WaveLength1.SPIN 
Book            PASM for beginners, Propeller 102
Section         3 Outputs
Programmer      Harprit Sandhu   
Date            Jan 20 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

--------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

var
  long  P_Val[4]               'variables we look at on the PST
  
OBJ
  fds : "FullDuplexSerial"     'for PST display
 
PUB wavelengths|x               'x becomes a local variable
fds.start(31,30,0,115200)       'start console at 115200 for debug output
waitcnt(clkfreq/100+cnt)        'wait for startup  
cognew(@gen_Waves, @P_Val)      'start new Cog to generate freq                                                              
waitcnt(clkfreq/100+cnt)        'wait for startup 
cognew(@Count_Waves, @P_Val)    'start new Cog to read wave count
waitcnt(clkfreq/60+cnt)         'wait for startup
  repeat                        'loop
    fds.tx($1)                  'home to 0,0
    fds.str(string("Data displays"))
    fds.tx($0d)                  'new line
    fds.str(string("1. Seeded value"))
    fds.tx($0d)                  'new line
    fds.str(string("2. Counter readings"))
    fds.tx($0d)                  'new line
    fds.str(string("3. Waves read"))
    fds.tx($0d)                  'new line
    fds.tx($0d)                  'new line
    fds.dec(P_Val[0])            'print value as decimal
    fds.tx($0d)                  'new line
    fds.dec(P_Val[1])            'print value as decimal 
    fds.str(string("    "))      'spaces
    fds.tx($0d)                  'new line
    fds.dec(P_Val[2])            'print value as decimal 
    fds.str(string("    "))      'spaces
    fds.tx($0d)                  'new line
    
    waitcnt(clkfreq/60+cnt)       'flicker free wait
    x:=x+1            'increment counter. (This routine clears up the screen )
    if x>100          'decision point     (by erasing extraneous lines of )
      fds.tx(16)      'clear screen       (bottom of PST display every 100 loops)
      x:=0            'reset counter      (needed only once in most cases.)
    P_Val[4]:=1       '1 allows PASM Cogs to start
'=============================================================================
'=============================================================================
dat       org     0                    'program generates a sq wave
gen_Waves mov     mem,    Par          ')
          add     mem,    #16          ')
          mov     temp,   #0           ')trap waits for P_Val[4] to = 1
          wrlong  temp,   mem          ')
:loop     rdlong  temp,   mem   wz     ')is P_Val[4]=0
if_z      jmp     #:loop               ')if 0 do again

          mov     dira,     set_dira    'to set DIRA  directions
          mov     mem,      par         'read par and store in mem
          wrlong  del_cnt,  mem         'store delay value used for wave
                             
out1      or      outa,     pin         'pin 0 high
          call    #clk_delay            'call delay
          andn    outa,     pin         'pin 0 low
          call    #clk_delay            'call delay       
          jmp     #out1                 'do again
'=========================== 
clk_delay mov     time,     del_cnt     'load del_cnt into time
out2      sub     time,     #1 wz       'sub 1 from time and set flag if 0
if_nz     jmp     #out2                 'if flag not 0 go back 
clk_delay_ret     ret                   'return 
'===========================
del_cnt    long   1_000                  'time of delay (freq delay)
set_dira   long   %00000000_00000000_00000000_00000001
pin        long   %00000000_00000000_00000000_00000001
mem       res     1
temp      res     1
fit
'=============================================================================
'=============================================================================
          org     0
Count_Waves    mov  mem5, Par          ')                                             
          add     mem5,   #16          ')
          mov     temp0,  #0           ')trap waits for P_Val[4] to = 1
          wrlong  temp0,  mem5         ')
:loop     rdlong  temp0,  mem5  wz     ')is P_Val[4]=0
if_z      jmp     #:loop               ')if 0 do again 

mov     dira,     set_dira1   'to set directions  
          mov     mem5,     par         'read par into mem5                                                                                        
          mov     mem6,     mem5         
          add     mem6,     #4          'set up mem6
          mov     mem7,     mem5
          add     mem7,     #8          'set up mem7 
          mov     mem8,     mem5
          add     mem8,     #12         'set up mem8
'
loop       mov    nmbr,     cnt         'read sys counter                                               
           add    nmbr,     del_long    'add time active delay to it
           mov    wave,     #0          'set wave counter to 0
           
loop2     call    #wait_lo          'pin 0 lo
          call    #wait_hi          'pin 0 hi
          mov     nmbr2,    cnt     'read sys counter 
          mov     nmbr3,    nmbr2   'read sys counter 
          add     wave,     #1      'add 1 to wave count
                               
          mov     Display5, nmbr    'counter1
          mov     Display6, nmbr3   'counter2  and difference
          mov     Display7, wave    'waves counted
          
          sub     nmbr2,    nmbr wc  'counter cycles used to here                                                                                
if_b      jmp     #loop2             'if not done do again.
         
          wrlong  Display6, mem6    '1
          wrlong  Display7, mem7    '2
          wrlong  Display8, mem8    '3
          call    #wait          
          jmp     #loop
'==============================
wait_lo   mov     temp, ina
          andn    temp, mask0  wz
if_nz     jmp     #wait_lo
wait_lo_ret       ret               'do it again
'==============================
wait_hi   mov     temp, ina
          andn    temp, mask0  wz
if_z      jmp     #wait_hi
wait_hi_ret       ret               'do it again
'==============================
wait      mov     temp, del_short
:loop     djnz    temp, #:loop
wait_ret          ret
'==============================
del_long  long    4_000_000     '1/20 sec
del_short long    4000
mask0     long    %11111111_11111111_11111111_11111110
set_dira1 long    %00000000_00000000_00000000_00000001
temp0     res     1
mem0      res     1
mem5      res     1
mem6      res     1
mem7      res     1
mem8      res     1
Display5  res     1
Display6  res     1
Display7  res     1
Display8  res     1
time      res     1
nmbr      res     1
nmbr2     res     1
nmbr3     res     1
wave      res     1
Fit