{{------------------------------------------------------------------------------
PASMSELF_LEVEL_TABLE.SPIN
Function of program
Create a self leveling table
This program addresses only one axis of the table
The other axis is essentially the same for the experimenter to add.

Program reads a signal from a Memsic 2125 and uses the information to
hold a table level as its base is tilted.  Two hobby R/C servos are used
as actuators.
Connections are as follows:
Servo red       5 volts
Servo black     Ground
Servo white     signal to servo from pin 22

Memsic 1   Temp out, not used
       2   Axis 1 to pin 0 on prop
       3   ground
       4   ground
       5   Axis 2 to pin 1 on prop
       6   5 volts power
See Book diagram and discussion


Program         PASMSELF_LEVEL_TABLE.SPIN
Book            PASM for beginners, Propeller 102
Section         Projects
Programmer      Harprit Sandhu
Date            Dec 21 2012

Revisions:      Report errors to harprit.sandhu@gmail.com




-------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
   
OBJ 
  fds : "FullDuplexSerial"      'for communications with PST
  r4p : "read4pots"             'reads pots  

VAR  
  long stack2[60]
  long stack3[30] 
  long value[10]   'values for detect routine
  long P_Val[8]    'values for potentiometers
  long Err,err_gain, Pres_Pos, Lev_Pos, Pot0, Range, e_spd 
   
PUB Main|x                        'x is a local variable here
fds.start(31,30,0,115200)         'start console at 115200 for debug output
r4p.Read4Pots(    @P_Val)         'read 4 pots
Cognew(@Detect,   @value)         'start PAMD Code
cognew(Feedback,  @stack3)        'start feedback cog
Lev_Pos:=  31_500      'Level position
Range:=      8000      'Range of travel for servo + and -
Err_gain:=    100      'gain of error signal
Pres_pos:=Value[2]     'reads present position value for servo
e_spd:=500             'error speed increment value
dira[0..1]~~           'set lines to outputs for the two servos.

  repeat        
    fds.tx($1)                    'home to 0,0
    fds.str(string("Data display"))     'print to display
    fds.str(string("       "))          'print 2 spaces to erase old data overflow
    fds.tx($d)                    'new line
    fds.tx($d)                    'new line

    fds.dec(||value[0]/16)        'first reading of cnt
    fds.str(string("     "))
    fds.tx($d)

     fds.dec(||value[1]/16)        'second reading of cnt
    fds.str(string("     ")) 
    fds.tx($d)
    
    fds.str(string("High = "))      
    fds.dec(||value[2]/16)          'length of time signal is high 
    fds.str(string("      "))                                                        
    fds.tx($d)                    'new line
    fds.tx($d)                    'new line

    fds.dec(||value[3]/16)          'third reading of cnt
    fds.str(string("     ")) 
    fds.tx($d) 
    'Feedback
    fds.str(string("Low  = "))
    fds.dec(||value[4]/16)           'length of time signal is high 
    fds.str(string("     "))
    fds.tx($d)                 
    fds.tx($d)
    fds.str(string("Total= "))       
    fds.dec(||value[2]/16+||value[4]/16 )     'wave length 
    fds.str(string("     "))
    fds.tx($d)              
    fds.tx($d)
    fds.str(string("Hi, Lo and Ttl have been divided by 16"))  'A note:
    fds.tx($d)
    fds.str(string("for viewing convenience."))
    fds.str(string("   "))
    waitcnt(clkfreq/60+cnt) 'flicker free wait
    x:=x+1         'increment counter.(This routine clears up the screen     )                
    if x>100       'decision point    (by erasing extraneous lines of bottom )                
      fds.tx(16)   'clear screen      (of PST display every 100 loops        )                
      x:=0         'reset counter

PUB feedback  
err:=0                            'reset error
 repeat                           'loop forever
    if ((||Value[2]/16)-25200)<0  'determine direction to move
       Pres_pos:=Lev_pos+err      'add error signal
       err:=err-e_spd             'increment error in neg direction
    else                          '
       Pres_pos:=Lev_pos+err      'add error signal
       err:=err+e_spd             'increment error in pos direction
    err<#=range                   'clamp error to less than range
    err#>=-range                  'clamp error to less than -range
    pres_Pos<#=(lev_pos+Range)    'clamp final position max
    pres_Pos#>=(lev_pos-Range)    'clamp final position min  
    value[6]:=pres_pos+ Pot0      'Pot0 is the first potentiometer                          
    waitcnt(clkfreq/20+cnt)       'wait to slow loop down
 
DAT       org     0  
Detect    mov     dira,     set_dira    'set direction for lines as inputs
          mov     mem,      par         'read the PAR register
          mov     mem1,     mem         'set up mem addresses
          add     mem1,     #4          'location 1
          mov     mem2,     mem
          add     mem2,     #8          'location 2
          mov     mem3,     mem
          add     mem3,     #12         'location 3
          mov     mem4,     mem
          add     mem4,     #16         'location 4 
          mov     mem5,     mem
          add     mem5,     #20         'location 5   
          mov     mem6,     mem
          add     mem6,     #24         'location 6 
'================================
loop      call    #wait_lo              'wait for signal to go low
          call    #wait_Hi              'wait for signal to go high
          mov     temp,     cnt         '
          wrlong  temp,     mem         '1st high

          call    #wait_lo              '
          mov     temp2,    cnt         '1st low
          wrlong  temp2,    mem1        '

          sub     temp2,    temp        'subtract the two
          wrlong  temp2,    mem2        'length of high

          call    #wait_Hi
          mov     temp3,    cnt         '2nd high completes wavelength
          wrlong  temp3,    mem3        '

          rdlong  temp2,    mem1        '
          sub     temp3,    temp2       'subtract for length of low
          wrlong  temp3,    mem4        'length of low
          mov     temp4,    temp3       '
          shr     temp4,    #3          'divide by 8 for use by servo (Scaling)
          call    #wait60               'wait 1/60 seconds
          call    #send_p               'send out pulse to position servo                                        '
          jmp     #loop                 'go back and do it again.
'=======================================================================
wait_Lo   mov     inbits,   ina
          andn    inbits,   mask23  wz
if_nz     jmp     #wait_lo
Wait_Lo_ret       ret
'=====================================
wait_Hi   mov     inbits,   ina
          andn    inbits,   mask23  wz
if_z     jmp     #wait_hi
Wait_Hi_ret       ret
'=====================================
wait60   mov delay, sixty
:loop   djnz  delay, :loop
wait60_ret        ret
'=====================================
send_P              
or   outa, Pin1 
rdlong short, mem6        
'add short, middle  
:loop    djnz short, :loop 
andn  outa, Pin1 
send_P_ret       ret
'=====================================
middle    long    31500        'middle position for servo
delay1    long    -17500       'correction for output pulse
sixty     long    100_000_     'long delay not critical
Set_dira  long    %00000000_00000000_00000000_00000011
mask23    long    %11111111_01111111_11111111_11111111
pin23     long    %11111111_11111111_11111111_11111110 
Pin1      long    %00000000_00000000_00000000_00000001

mem       res 1
mem1      res 1
mem2      res 1
mem3      res 1
mem4      res 1
mem5      res 1
mem6      res 1
temp      res 1
temp2     res 1
temp3     res 1
temp4     res 1
inbits    res 1
delay     res 1
short     res 1
Fit