{{-------------------------------------------------------------------------------
                                                                                      
PASM_BookMetronome.SPIN
Function of program
Makes an electronic Metronome.

Notes:  Program reads a potentiometer and uses its value to
annunciate from 40 to 210 beats on a speaker.
An MCP 3208 is installed at pins 24..27.
Speaker is on line 7
Also use lines 0 and 1 to read and generate signals.
Signals are hard to see on an Oscilloscope because scan
is very slow.
See discussion in book

Program         PASM_BookMetronome.SPIN
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Jan 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

-------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000  
  vars=8
   
OBJ 
  fds : "FullDuplexSerial"  'for communications with PST
  r4p : "read4Pots"         'to read potentiometers
  
VAR  
  long P_Val[vars] 
  byte subsc
  
PUB Main|x 
fds.start(31,30,0,115200)       'start console at 115200 for debug output '
r4p.Read4Pots(@P_Val)           '
waitcnt(clkfreq/100+cnt)        'wait for startup '
Cognew(@beats,        @P_Val)   
waitcnt(clkfreq/100+cnt)        'wait for startup '                                                         
Cognew(@chirp,       @P_Val)
dira[1]~~                       'set lines to inputs for the two signals.
dira[7]~~                       'set lines to inputs for the two signals.
  repeat                        '
    fds.tx($1)                  'home to 0,0
    fds.str(string("Data display"))  'print to display
    fds.str(string("   "))      'spaces to erase old data overflow
    fds.tx($d)                  'new line
    fds.tx($d)                  'new line
    repeat subsc from 0 to vars-1
      if subsc==4 
        fds.tx($d)              'new line spaces first 4 values apart 
      fds.dec(subsc)                            
      fds.str(string("   "))    '
      fds.dec(P_Val[subsc])     'print value
      fds.str(string("   "))    '
      fds.tx($d)                'new line
    fds.tx($d)                  'new line                                             
    fds.str(string("4 potentiometers and"))  'print to display
    fds.tx($d)                  'new line 
    fds.str(string("rest of misc variables"))'print to display 
    fds.tx($d)                  'new line 
    fds.str(string("Beats = "))'print to display
    fds.dec((P_Val[0]+19)/24+40)
    P_Val[4]:=600_000_000/ ((P_Val[0]+19)/24+40)
    waitcnt(clkfreq/60+cnt)       'flicker free wait
    x:=x+1          'increment counter. (This routine clears up the screen)
    if x>100         'decision point     (by erasing extraneous lines of bottom)
      fds.tx(16)    'clear screen       (of PST display every 100 loops)
      x:=0          'reset counter

'=============================================================================
'=============================================================================
DAT       org     0
Beats     mov     dira,     set_Dira    'Set up DIRA
          mov     mem,      par         'Loc 0
          mov     mem4,     mem         '
          add     mem4,     #16         'Loc 4
          
:re_read  rdlong  reading,  mem4        'Read location 0, Pot(0)                                                                   
          mov     B_delay,  reading     'copy reading to B_delay
          add     B_delay,  #1          'so it is always > 0
          or      outa,     pin1        'pin1 high
          call    #Beat_delay           'call Beat delay
          andn    outa,     pin1        'pin1 low
          call    #Beat_delay           'call beat delay
          jmp     #:re_read             'loop back and read again
'======================================
Beat_delay                              'delay between beats
          mov     pause,   B_delay      'copy into pause
:loop     djnz    pause,   #:loop       'pause loop
Beat_delay_ret     ret
'=======================================
set_dira  long %00001011_00000000_00000000_10000010
pin0      long %00000000_00000000_00000000_00000001
pin1      long %00000000_00000000_00000000_00000010
mem       res 1
mem4      res 1
B_delay   res 1
reading   res 1
pause     res 1
temp      res 1
fit
