{{-------------------------------------------------------------------------------                                                                                      
PASM_Book_Metronome.SPIN
Function of program
Towards making an electronic Metronome.

Notes:  Program reads a potentiometer and uses its value to
annunciate fromm 40 to 210 beats on a speaker.
An MCP 3208 is installed at pins 24..27.
Speaker is placed on line 1
Signals are hard to see on an Oscilloscope because scan
is very slow.
See discussion in book

Program         PASM_Book_Metronome.SPIN
Book            PASM for beginners, Propeller 102
Section         Outputs
Programmer      Harprit Sandhu   
Date            Jan 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

-------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000  
  vars=8

OBJ 
  fds : "FullDuplexSerial"  'for communications with PST
  r4p : "read4Pots"         'to read potentiometers
  
VAR  
  long P_Val[vars] 
  byte subsc
  
PUB Main|x 
fds.start(31,30,0,115200)       'start console at 115200 for debug output '
r4p.Read4Pots(@P_Val)
Cognew(@beats,        @P_Val)                                                                                                            
Cognew(@chirp,       @P_Val)
  repeat                        '
    fds.tx($1)                  'home to 0,0
    fds.str(string("Data display"))  'print to display
    fds.str(string("   "))      'spaces to erase old data overflow
    fds.tx($d)                  'new line
    fds.tx($d)                  'new line
    repeat subsc from 0 to vars-1
      if subsc==4 
        fds.tx($d)              'new line spaces first 4 values apart 
      fds.dec(subsc)
      fds.str(string("   "))    '
      fds.dec(P_Val[subsc])     'print value
      fds.str(string("   "))    '
      fds.tx($d)                'new line
    fds.tx($d)                  'new line                                             
    fds.str(string("4 potentiometers and"))  'print to display
    fds.tx($d)                  'new line 
    fds.str(string("rest of misc variables"))'print to display 
    fds.tx($d)                  'new line 
    fds.str(string("Beats = "))'print to display
    fds.dec((P_Val[0]+19)/24+40)
    P_Val[4]:=600_000_000/ ((P_Val[0]+19)/24+40)
    waitcnt(clkfreq/60+cnt)       'flicker free wait
    x:=x+1          'increment counter. (This routine clears up the screen )
    if x>10         'decision point     (by erasing extraneous lines of bottom )
      fds.tx(16)    'clear screen       (of PST display every 10 loops )
      x:=0          'reset counter
    P_Val[5]:=1     'Allows PASM programs to run when this becomes 1, was 0
'=============================================================================
'=============================================================================
DAT       org     0
Beats     mov     mem,    Par           ')
          add     mem,    #20           ')
          mov     temp,   #0            ')trap waits for PVal[5] to = 1
          wrlong  temp,   mem           ')
:loop7    rdlong  temp,   mem   wz      ')
if_z      jmp     #:loop7               ')
              
          mov dira, set_Dira            'Sets DIRA
          mov     mem,      par         'Loc 0
          mov     mem4,     mem         '
          add     mem4,     #16         'Loc 4
          
:re_read  rdlong  reading,  mem4        'Read location 4, Pot(0) from SPIN                                        '                          
          mov     B_delay,  reading     'copy reading to B_delay
          add     B_delay,  #1          'so it is always > 0
          or      outa,     pin1        'pin1 high
          call    #Beat_delay           'call Beat delay
          andn    outa,     pin1        'pin1 low
          call    #Beat_delay           'call beat delay
          jmp     #:re_read             'loop back and read again
'==================
Beat_delay                              'delay between beats
          mov     pause,   B_delay      'copy into pause
:loop     djnz    pause,   #:loop       'pause loop
Beat_delay_ret    ret                   '
'===================
set_dira  long %00001011_00000000_00000000_10000010
pin1      long %00000000_00000000_00000000_00000010
mem       res 1
mem4      res 1   
B_delay   res 1  
reading   res 1
pause     res 1 
temp      res 1
FIT
'=============================================================================
'=============================================================================
          org     0                    'all objects must start with this                                                                 
Chirp     mov     mem0,    Par         ')
          add     mem0,    #20         ')
          mov     temp0,  #0           ')trap waits for PVal[5] to = 1
          wrlong  temp0,  mem0         ')
:loop9    rdlong  temp0,  mem0  wz     ')
if_z      jmp     #:loop9              ')

          mov     dira,     set_Dirb    'sets Dira
begin     mov     numb,     rChirps     'times to toggle chirp line
          call    #wait4low             'wait for beat signal low
          call    #wait4high            'wait for best signal high
:loop     or      outa, pin7            'pin7 high

          call    #S_delay              'delay between chirp toggle
          andn    outa,     pin7        'pin7 low
          call    #S_delay              'delay between chirp toggle
          sub     numb,     #1  wz      'decrement chipr counter
if_nz     jmp     #:loop                'if not done do it again                                                                  
          jmp     #begin                'go wait for next high
'=================
S_delay                                 'short delay between beats
          mov     paused,   sdelay      'load counter
:loop     djnz    paused,   #:loop      'decrement loop
S_delay_ret       ret                   'return
'====================
wait4high                               'wait for signal to go high
           mov    temp0,    ina         'load pin I/O conditions
           andn   temp0,    mask1 wz    'mask for pin 0
if_z       jmp    #wait4high            'do again if not 1
wait4high_ret     ret                   'return
'====================
wait4low                                'wait for signal to go low
           mov    temp0,    ina         'load pin I/O conditions
           andn   temp0,    mask1 wz    'mask for pin 0 
if_nz      jmp    #wait4low             'do again if not 0
wait4low_ret      ret                   'return
'====================
rChirps   long 4
set_dirb  long %00001011_00000000_00000000_10000010
pin7      long %00000000_00000000_00000000_10000000
mask1     long %11111111_11111111_11111111_11111101
sdelay    long 4000
ldelay2   long 200000
paused    res     1
numb      res     1
mem0      res     1
temp0     res     1
FIT
