{{-------------------------------------------------------------------------------
PASM_Book_LCD_2x16.SPIN
Function of program
Implements the use of a 2 line by 16 character
in PASM. Data is written to the LCD in PASM or through the HUB memory.
View data on PST
See book for details and operation scheme

Connections are as follows
Prop    LCD
        16  Ground                   for LED
        15  5 volts through 220 ohms for LED lighting
        
0       14   data buss
1       13      "
2       12      "
3       11      "
4       10      "
5       9       "
6       8       "
7       7    data buss
      
20      6    ENable
21      5    Read Write
22      4    Reg Select
        3    Ground
        2    5 volt     Power Logic
        1    Ground

Program         PASM_Book_LCD_2x16.SPIN
Book            PASM for beginners, Propeller 102
Section         Projects
Programmer      Harprit Sandhu
Date            Feb 10 2013

Revisions:      Report errors to harprit.sandhu@gmail.com

--------------------------------------------------------------------------------}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  Vars =32     'hub variables to be displayed
  
VAR
  long  stack1[35],line1,line2,temp9
  long  P_Val[vars] ,alpha,beta,theta,gamma,addr
  byte  subscr,nu
  
OBJ
  fds : "FullDuplexSerial"
  r4p : "Read4Pots"

PUB Go|x 
waitcnt(clkfreq/20+cnt) 
r4p.Read4Pots(@P_Val) 
Cognew(@display,  @P_val) 
fds.start(31,30,0,115200)       'start PST console at 115200 for output 
'Cognew(@Write_LCD, @P_val)
waitcnt(clkfreq/20+cnt)
 repeat
  '{{       
   fds.tx(1)                    'home 
   fds.str(string("Displaying Hub variables")) 'heading                                                                                              
   fds.str(string("  "))        'spaces to erase old data overflow  
   fds.tx($d)                   'new line      
   fds.tx($d)                   'new line                                 
'===========================================================================
'===========================================================================
'===========================================================================
   'the following lines of code set up a dummy display for development
   'purposes.  These will be blank in the final set up and will be
   'written over by other cogs as needed to display data.
   'comment this code out if you want to run the lower code
   'Run only one set at a time.
  '{{
   P_Val[4]:=$30303030         '0000
   P_Val[5]:=$31313131         '1111 
   P_Val[6]:=$32323232         '2222       Line 1                     
   P_Val[7]:=$33333333         '3333                                  
   P_Val[8]:=$34343434         '4444    x  not used for 16x2          
   P_Val[9]:=$35353535         '5555    x  but allows up to 40x2      
   P_Val[10]:=$36363636        '6666    x      "                      
   P_Val[11]:=$37373737        '7777    x      "                                                                       
   P_Val[12]:=$38383838        '8888    x      "                      
   P_Val[13]:=$39393939        '9999    x      "          
                                        
   P_Val[14]:=$41414141        'AAAA                                  
   P_Val[15]:=$42424242        'BBBB                                  
   P_Val[16]:=$43434343        'CCCC         Line 2                   
   P_Val[17]:=$44444444        'DDDD                                  
   P_Val[18]:=$45454545        'EEEE     x       "                    
   P_Val[19]:=$46464646        'FFFF     x       "                    
   P_Val[20]:=$47474747        'GGGG     x       "                    
   P_Val[21]:=$48484848        'HHHH     x       "                    
   P_Val[22]:=$49494949        'IIII     x       "                    
   P_Val[23]:=$50505050        'JJJJ     x       "                    
  ' }}                                                                       
'===========================================================================
'===========================================================================
'===========================================================================
   'The folowing llines of code display "Hello World" on the display
   'The are currently commented out.  Decide if you want to display the
   'upper code or this code and run the program.
 {{
  repeat    
     nu:=0      
     line1:=string("Hello           ")
     repeat subscr from 4 to 7 
        alpha:=byte [line1] [nu]
        beta:=byte [line1] [nu+1]
        Theta:=byte [line1] [nu+2]
        Gamma:=byte [line1] [nu+3]
        alpha<<=8
        alpha:=alpha ^ beta
        alpha<<=8
        alpha:=alpha ^ theta
        alpha<<=8
        alpha:=alpha ^ gamma
        p_val[subscr]:=alpha
        alpha:=" "
        nu:=nu+4
     nu:=0
     line1:=string("world            ")
     repeat subscr from 14 to 17
        alpha:=byte [line1] [nu]
        beta:=byte [line1] [nu+1]
        Theta:=byte [line1] [nu+2]
        Gamma:=byte [line1] [nu+3]
        alpha<<=8
        alpha:=alpha ^ beta  
        alpha<<=8
        alpha:=alpha ^ theta
        alpha<<=8
        alpha:=alpha ^ gamma
        p_val[subscr]:=alpha
        alpha:=" "
        nu:=nu+4
        }}
'===========================================================================
'===========================================================================
'===========================================================================
    'following code display data o the PST. You can comment it out if you like.
     fds.tx(16) 
     repeat subscr from 0 to vars 'loop to display all vars 
       if subscr==4             'separate out first 4 POT values
         fds.tx($d)             'separator line  
       fds.hex(P_Val[subscr], 8) 'print to the PST in binary                                                                                                              
       fds.str(string(" "))     'two spaces to erase old data overflow 
       if subscr<10             'align values under 10
        fds.str(string(" "))    'alignment  space                                                                                
       fds.dec(subscr)          'print value as decimal value                                                                                                                
       fds.str(string(" "))     'spaces to erase old data overflow
       fds.dec(P_Val[subscr])   'print to the PST in binary                                                                                                              
       fds.tx($d)                
    
     waitcnt(clkfreq/60+cnt)    'wait 1/60.   
     x:=x+1           'increment counter. (This routine clears up the screen    )
     if x>100         'decision point     (by erasing extraneous lines of       )
       fds.tx(16)     'clear screen       (bottom of PST display every 100 loops)
       x:=0           'reset counter      (Needed only once in most cases.      )
 
DAT  
          org     0         
display   mov     mem,      par        'Read Par into mem                                       '
          add     mem1,     #16        'Set Mem1
          mov     mem2,     mem        '
          add     mem2,     #20        'Set Mem2 
          mov     mem3,     mem        '
          add     mem3,     #24        'Set Mem3 
          mov     mem4,     mem        '
          add     mem4,     #28        'Set Mem4 
          mov     dira,     set_dir    'Set directions
          call    #Init_LCD            'call initialize routine for LCD
          mov     data, half_sec       'load half sec
          call    #wait                'wait for start up of LCD
read_hub '===========================
          mov     mem1,     mem        'read  base address       
          add     mem1,     #12        'base address for looping
          mov     nmbr,     #20        'number of locations needed
:loop     add     mem1,     #4         'next mem location
          rdlong  data, mem1           'read base Hub mem location
          call    #readFour            'unpack next for values
          sub     nmbr,     #1 wz      'decrement n for count to completion
if_nz     jmp     #:loop               'do again if not done
park      jmp     #park                'or
'          jmp    #read_hub             'if you want to loop
'===================================== '
readFour  rdlong  data, mem1            'Read base address
          ror     data, #24             'roll right 24
          andn    data, mask8           'mask off all but right 8 bits
          call    #send_char            'send out what was read as ASCII code 
          rdlong  data, mem1            'read first Hub mem location again
          ror     data, #16             'roll right 16
          andn    data, mask8           'mask off all but right 8 bits 
          call    #send_char            'send out what was read as ASCII code
          rdlong  data, mem1            'read first Hub mem location
          ror     data, #8              'roll right 8
          andn    data, mask8           'mask off all but right 8 bits 
          call    #send_char            'send out what was read as ASCII code
          rdlong  data, mem1            'read first Hub mem location
          ror     data, #0              'shift right not needed now
          andn    data, mask8           'mask off all but right 8 bits 
          call    #send_char            'send out what was read as ASCII code
readfour_ret RET                        'return
'subroutines=====================================================================
Init_LCD  andn    outa,     setRw       'RW set low
          andn    outa,     setRs       'Rs set low
          andn    outa,     setEN       'EN set low
          mov     data,    #(%0011_0000) 'load instruction
          call    #send_Instr           'send out instruction
          mov     data,   delay1        'load first delay
          call    #wait                 'wait  
          mov     data,    #(%0011_0000)'load instruction 
          call    #send_Instr           'send out instruction
          mov     data,   delay2        'load second delay
          call    #wait                 'wait 
          mov     data,    #(%0011_0000)'load instruction 
          call    #send_Instr           'send out instruction
          mov     data,   delay3        'load third delay
          call    #wait                 'wait
          '======================       '
          mov     data,    #(%0011_1000)  'clears blocks
          call    #send_Instr           'load instruction
          call    #wait_2ms             'wait 
          mov     data,    #(%0000_1111) 'cursor etc set
          call    #send_Instr           'load instruction
          call    #wait_2ms             'wait  
          mov     data,    #(%0000_0001) 'clears display
          call    #send_Instr           'load instruction
          call    #wait_2ms             'wait  
          mov     data,    #(%0000_0010) 'go home
          call    #send_Instr           'load instruction
          call    #wait_2ms             'wait 
Init_LCD_ret          ret               'Return                                        '
'=====================           '
wait      mov     temp,     data        'load passed data
:loop     djnz    temp,     #:loop      'wait loop
wait_ret  ret                           'return
'=====================            ' '
wait_2ms  mov     temp,     d2ms        'load 2 ms
:loop     djnz    temp,     #:loop      'wait loop
wait_2ms_ret      ret                   'return
'=====================               '
Send_Instr call   #busy                 'Check busy flag
          andn    outa,     setRw       'RW set low 
          andn    outa,     setRs       'RS set low
          mov     outa,     data        'load outa 
          or      outa,     setEN       'set EN hi 
          andn    outa,     setEN       'set EN lo 
Send_Instr_ret    ret                   'retuirn
'=====================
Send_Char call    #busy                 '
          andn    outa,     setRw       'RW set low
          or      outa,     setRs       'RS set hi 
          andn    outa,     mask_clr    'load outa
          or      outa,     data
          call    #wait_2ms             'wait 2 ms 
          or      outa,     setEN       'set EN hi                                                    
          andn    outa,     setEN       'set EN lo
Send_Char_ret     ret                   'reutrn
'=====================
Busy      mov     dira,     Set_Inp     'change to input byte
          or      outa,     setRw       'RW set hi
          andn    outa,     setRs       'RS set lo                                        '                 
:loop     mov     temp,     ina         'read input lines                                                   
          andn    temp,     mask7 wz    'look at busy bit
   if_nz  jmp     #:loop                'loop back if not a 0    
          call    #wait_2ms             'wait 2 ms 
          mov     dira,     Set_Dir     'set back to output lines
Busy_ret          ret                   'return
'constants============
half_sec long   40_000_000  '0.5 sec    
delay1    long      32_000  '4.1 milli
delay2    long       4_000  '100 micro
delay3    long      80_000  '1 milli
d2ms      long         400   '2 ms
set_Dir   long  %00000000_11111111_00000000_11111111   'set for output
set_Inp   long  %00000000_00000000_00000000_00000000   'set for input                                                      
setEN     long  %00000000_00010000_00000000_00000000                                                                       
setRW     long  %00000000_00100000_00000000_00000000                                                                      
setRS     long  %00000000_01000000_00000000_00000000
mask7     long  %11111111_11111111_11111111_01111111   'mask for pin 7 BUSY bit
mask_clr long   %00000000_00000000_00000000_11111111   'clear byte
mask8     long  %11111111_11111111_11111111_00000000   'mask for byte
'variables===============                          
temp      res     1
mem       res     1
mem1      res     1
mem2      res     1
mem3      res     1
mem4      res     1
mem5      res     1
data      res     1
nmbr      res     1
Fit
